@@ -129,7 +129,7 @@ class ANN(MatchFilter):
         query_vector: str,
         hits: int,
         label: str,
-        approximate: bool = True
+        approximate: bool = True,
     ) -> None:
         """
         Match documents according to the nearest neighbor operator.
@@ -151,10 +151,8 @@ class ANN(MatchFilter):
         self._approximate = "true" if self.approximate is True else "false"
 
     def create_match_filter(self, query: str) -> str:
-        return (
-            '([{{"targetNumHits": {}, "label": "{}", "approximate": {}}}]nearestNeighbor({}, {}))'.format(
-                self.hits, self.label, self._approximate, self.doc_vector, self.query_vector
-            )
+        return '([{{"targetNumHits": {}, "label": "{}", "approximate": {}}}]nearestNeighbor({}, {}))'.format(
+            self.hits, self.label, self._approximate, self.doc_vector, self.query_vector
         )
 
     def get_query_properties(self, query: Optional[str] = None) -> Dict[str, str]:
@@ -206,20 +204,26 @@ class RankProfile(object):
 class QueryModel(object):
     def __init__(
         self,
+        name: str = "default_name",
         query_properties: Optional[List[QueryProperty]] = None,
         match_phase: MatchFilter = AND(),
         rank_profile: RankProfile = RankProfile(),
+        body_function: Optional[Callable[[str], Dict]] = None,
     ) -> None:
         """
         Define a query model.
 
+        :param name: Name of the query model. Used to tag model related quantities, like evaluation metrics.
         :param query_properties: Optional list of QueryProperty.
         :param match_phase: Define the match criteria. One of the MatchFilter options available.
         :param rank_profile: Define the rank criteria.
+        :param body_function: Function that take query as parameter and returns the body of a Vespa query.
         """
+        self.name = name
         self.query_properties = query_properties if query_properties is not None else []
         self.match_phase = match_phase
         self.rank_profile = rank_profile
+        self.body_function = body_function
 
     def create_body(self, query: str) -> Dict[str, str]:
         """
@@ -229,6 +233,10 @@ class QueryModel(object):
         :return: dict representing the request body.
         """
 
+        if self.body_function:
+            body = self.body_function(query)
+            return body
+
         query_properties = {}
         for query_property in self.query_properties:
             query_properties.update(query_property.get_query_properties(query=query))
@@ -247,29 +255,3 @@ class QueryModel(object):
         return body
 
 
-class VespaResult(object):
-    def __init__(self, vespa_result, request_body=None):
-        self._vespa_result = vespa_result
-        self._request_body = request_body
-
-    @property
-    def request_body(self) -> Optional[Dict]:
-        return self._request_body
-
-    @property
-    def json(self) -> Dict:
-        return self._vespa_result
-
-    @property
-    def hits(self) -> List:
-        return self._vespa_result.get("root", {}).get("children", [])
-
-    @property
-    def number_documents_retrieved(self) -> int:
-        return self._vespa_result.get("root", {}).get("fields", {}).get("totalCount", 0)
-
-    @property
-    def number_documents_indexed(self) -> int:
-        return (
-            self._vespa_result.get("root", {}).get("coverage", {}).get("documents", 0)
-        )
